#include "switch_panel.h"
#include "hilink.h"

//====================== 按键检测相关变量 ======================
static bool g_key_states[SWITCH_COUNT] = {true, true, true, true};  // 上一次按键状态
static uint32_t g_key_press_time[SWITCH_COUNT] = {0};               // 按键按下开始时间
static bool g_key_debounce_flag[SWITCH_COUNT] = {false};            // 防抖标志
static bool g_is_long_press_handled[SWITCH_COUNT] = {false};        // 长按处理标志

// 定时器参数
typedef struct {
    int key_id;
} key_timer_param_t;

static key_timer_param_t g_timer_params[SWITCH_COUNT];

//====================== 开关控制函数 ======================

// 更新单个开关状态
void update_switch_state(int switch_id, bool state) {
    if (switch_id < 0 || switch_id >= SWITCH_COUNT) {
        e_printf("无效的开关ID: %d\r\n", switch_id);
        return;
    }
    
    // 检查总开关是否允许操作
    if (!g_persistent_state.master_switch && state) {
        e_printf("总开关关闭，不允许开启开关%d\r\n", switch_id + 1);
        return;
    }
    
    // 更新开关状态
    if (g_persistent_state.switches[switch_id].switch_on != state) {
        g_persistent_state.switches[switch_id].switch_on = state;
        
        // 更新LED指示灯(开关开启时LED白灯，关闭时LED黄灯)
        g_persistent_state.switches[switch_id].led_state = state;
        
        // 同步硬件状态
        set_switch_output(switch_id, state);
        set_led_output(switch_id, state ? LED_WHITE : LED_YELLOW);
        
        e_printf("开关%d 状态更新: %s\r\n", 
               switch_id + 1, state ? "开" : "关");
        
        // 立即保存状态
        save_persistent_state();
        
        // 上报状态给云端
        if (g_persistent_state.is_bound) {
            fast_report_switch(switch_id);
        }
    }
}

// 更新总开关状态
void update_master_switch(bool state) {
    if (g_persistent_state.master_switch != state) {
        g_persistent_state.master_switch = state;
        
        e_printf("总开关状态更新: %s\r\n", state ? "开" : "关");
        
        // 应用总开关控制逻辑
        apply_master_switch_control();
        
        // 立即保存状态
        save_persistent_state();
        
        // 上报状态给云端
        if (g_persistent_state.is_bound) {
            fast_report_master_switch();
        }
    }
}

// 应用总开关控制逻辑
void apply_master_switch_control(void) {
    
    if (!g_persistent_state.master_switch) {
        // 总开关关闭时，强制关闭所有子开关
        e_printf("总开关关闭，强制关闭所有子开关\r\n");
        
        for (int i = 0; i < SWITCH_COUNT; i++) {
            // 如果子开关之前是开着的，需要同步状态
            if (g_persistent_state.switches[i].switch_on) {
                g_persistent_state.switches[i].switch_on = false;
                g_persistent_state.switches[i].led_state = false;
                e_printf("强制关闭子开关%d\r\n", i + 1);
            }
            
            // 更新硬件状态
            set_switch_output(i, false);
            set_led_output(i, LED_YELLOW);
        }
    } else {
        // 总开关开启时，恢复各开关的原状态（不改变子开关状态）
        // e_printf("总开关开启，恢复各子开关原有状态\r\n");
        
        // for (int i = 0; i < SWITCH_COUNT; i++) {
        //     // 硬件状态跟随子开关的实际状态
        //     set_switch_output(i, g_persistent_state.switches[i].switch_on);
        //     set_led_output(i, g_persistent_state.switches[i].led_state ? LED_WHITE : LED_YELLOW);
        // }
    }
    // 立即保存状态
    save_persistent_state();
    
    // 同步所有子开关状态到云端
    if (g_persistent_state.is_bound) {
        fast_report_all_switches_async();
    }
    
    e_printf("所有子开关状态已同步\r\n");
}

//====================== 按键检测与处理 ======================

// 按键防抖定时器回调
void key_debounce_timer_callback(uintptr_t data) {
    key_timer_param_t *param = (key_timer_param_t *)data;
    if (!param) return;
    
    int key_id = param->key_id;
    if (key_id < 0 || key_id >= SWITCH_COUNT) return;
    
    // 清除防抖标志
    g_key_debounce_flag[key_id] = false;
}

// 按键扫描任务
int key_scan_task(void *arg) {
    (void)arg;
    
    e_printf("[KEY] 按键扫描任务开始\r\n");
    
    while (!osal_kthread_should_stop()) {
        // 扫描所有按键
        for (int i = 0; i < SWITCH_COUNT; i++) {
            bool current_state = get_key_input(i);
            uint32_t current_time = hfsys_get_time();
            
            // 检测按键状态变化
            if (current_state != g_key_states[i] && !g_key_debounce_flag[i]) {
                if (current_state == KEY_PRESSED) {
                    // 按键按下
                    g_key_press_time[i] = current_time;
                    g_is_long_press_handled[i] = false;
                    
                    e_printf("[KEY] 按键%d 按下\r\n", i + 1);
                } else {
                    // 按键松开
                    uint32_t press_duration = current_time - g_key_press_time[i];
                    
                    if (press_duration >= KEY_LONG_PRESS_MS && !g_is_long_press_handled[i]) {
                        // 长按事件（如果在按下期间没有处理）
                        handle_key_long_press(i);
                        g_is_long_press_handled[i] = true;
                    } else if (press_duration < KEY_LONG_PRESS_MS) {
                        // 短按事件
                        handle_key_press(i);
                    }
                    
                    e_printf("[KEY] 按键%d 松开 (持续时间: %ums)\r\n", i + 1, press_duration);
                }
                
                // 启动防抖定时器
                g_key_debounce_flag[i] = true;
                if (g_key_debounce_timer[i]) {
                    uapi_timer_start(g_key_debounce_timer[i], 
                                   KEY_DEBOUNCE_MS * 1000, 
                                   key_debounce_timer_callback, 
                                   (uintptr_t)&g_timer_params[i]);
                }
                
                g_key_states[i] = current_state;
                g_runtime_state.switches[i].physical_key = current_state;
            }
            
            // 检查是否达到长按时间（在按下期间监测）
            if (current_state == KEY_PRESSED && !g_is_long_press_handled[i]) {
                uint32_t press_duration = current_time - g_key_press_time[i];
                if (press_duration >= KEY_LONG_PRESS_MS) {
                    handle_key_long_press(i);
                    g_is_long_press_handled[i] = true;
                }
            }
        }
        
        // 休眠一段时间再继续扫描
        osal_msleep(10);  // 10ms扫描间隔
    }
    
    return 0;
}

// 处理按键短按事件
void handle_key_press(int key_id) {
    if (key_id < 0 || key_id >= SWITCH_COUNT) {
        return;
    }
    
    e_printf("[KEY] 处理按键%d 短按事件\r\n", key_id + 1);
    
    // 在配网模式下，忽略短按事件
    if (g_runtime_state.mode == MODE_CONFIG) {
        e_printf("[KEY] 配网模式下，忽略短按事件\r\n");
        return;
    }
    
    // 在产测模式下，忽略短按事件
    if (g_runtime_state.mode == MODE_FACTORY_TEST) {
        e_printf("[KEY] 产测模式下，忽略短按事件\r\n");
        return;
    }
    
    // 正常模式下切换开关状态
    bool current_state = g_persistent_state.switches[key_id].switch_on;
    update_switch_state(key_id, !current_state);
}

// 处理按键长按事件
void handle_key_long_press(int key_id) {
    if (key_id < 0 || key_id >= SWITCH_COUNT) {
        return;
    }
    
    e_printf("[KEY] 处理按键%d 长按事件\r\n", key_id + 1);
    
    // 只有第一个按键支持长按进入配网模式
    if (key_id == 0) {
        e_printf("[KEY] 长按第一个按键，检查配网条件\r\n");
        
        // 只有在正常模式下且设备未绑定时才能进入配网模式
        if (g_runtime_state.mode == MODE_NORMAL && !g_persistent_state.is_bound) {
            extern int g_config_key_id;
            g_config_key_id = key_id;  // 设置触发配网的按键ID
            enter_config_mode();
        } else {
            if (g_persistent_state.is_bound) {
                e_printf("[KEY] 设备已绑定，不能进入配网模式\r\n");
            } else {
                e_printf("[KEY] 非正常模式，不能进入配网模式\r\n");
            }
        }
    } else {
        e_printf("[KEY] 非第一个按键的长按，忽略\r\n");
    }
}

//====================== 初始化函数 ======================

int key_system_init(void) {
    int ret = ERRCODE_SUCC;
    
    // 初始化定时器
    ret = uapi_timer_init();
    if (ret != ERRCODE_SUCC) {
        e_printf("[KEY] 定时器初始化失败: %d\r\n", ret);
        // return HF_FAIL;
    }
    ret = uapi_timer_adapter(TIMER_INDEX_1, TIMER_1_IRQN, 1);
    if (ret != 0) {
        e_printf("定时器适配器初始化失败\r\n");
        return HF_FAIL;
    }
    // 初始化定时器参数
    for (int i = 0; i < SWITCH_COUNT; i++) {
        g_timer_params[i].key_id = i;
        
        // 创建防抖定时器
        ret = uapi_timer_create(TIMER_INDEX_1, &g_key_debounce_timer[i]);
        if (ret != ERRCODE_SUCC) {
            e_printf("[KEY] 创建按键%d防抖定时器失败: %d\r\n", i + 1, ret);
            return HF_FAIL;
        }
        
        // 初始化按键状态
        g_key_states[i] = get_key_input(i);
        g_runtime_state.switches[i].physical_key = g_key_states[i];
    }
    
    // 创建按键扫描任务
    g_key_scan_task_handle = osal_kthread_create((osal_kthread_handler)key_scan_task, 
                                                NULL, 
                                                "key_scan", 
                                                TASK_STACK_SIZE);
    if (!g_key_scan_task_handle) {
        e_printf("[KEY] 创建按键扫描任务失败\r\n");
        return HF_FAIL;
    }
    
    // 设置任务优先级
    ret = osal_kthread_set_priority(g_key_scan_task_handle, TASK_PRIORITY_HIGH);
    if (ret != 0) {
        e_printf("[KEY] 设置按键扫描任务优先级失败: %d\r\n", ret);
    }
    
    e_printf("[KEY] 按键系统初始化完成\r\n");
    return HF_SUCCESS;
}

// 快速上报单个开关状态(兼容旧接口，内部使用异步上报)
void fast_report_switch(int switch_id) {
    fast_report_switch_async(switch_id);
}

// 快速上报总开关状态(兼容旧接口，内部使用异步上报)
void fast_report_master_switch(void) {
    fast_report_master_switch_async();
}